-----DEBUG CONTROL--------------------------------------------------------------

debug_output = true -- true/false --display debug messages in the console

-----DISPLAY CONSOLE MESSAGE----------------------------------------------------

-- Display a message in the console for debugging, message can be a number variable
-- or a string but not mixed. No line feed character is added.
function Msg(value)
  --returns nothing

  if debug_output then --check if flag is set true (at the start of this script)
    reaper.ShowConsoleMsg(tostring(value).."\n")
  end
end


-- NoIndex: true

--[[
	Lokasenna_GUI example

	- Getting user input before running an action; i.e. replacing GetUserInputs

]]--

--------------------------------
 --REATRAK LOCAL GUI LIBRARY start
--------------------------------
local info = debug.getinfo(1,'S');
script_path = info.source:match[[^@?(.*[\/])[^\/]-$]]
--Msg("script_path "..script_path)
 local lib_path = script_path --reaper.GetExtState("Lokasenna_GUI", "lib_path_v2")
--if not lib_path or lib_path == "" then
    --reaper.MB("Couldn't load the ReaTrak Lokasenna_GUI library. Please check Scripts/ReaTrak/ReaTrak_Classes/", "Whoops!", 0)
    --return
--end

loadfile(script_path .. "ReaTrak_Core.lua")()

GUI.req(script_path .. "ReaTrak_Classes/Class - Label.lua")()
GUI.req(script_path .. "ReaTrak_Classes/Class - Knob.lua")()
GUI.req(script_path .. "ReaTrak_Classes/Class - Tabs.lua")()
GUI.req(script_path .. "ReaTrak_Classes/Class - Slider.lua")()
GUI.req(script_path .. "ReaTrak_Classes/Class - Button.lua")()
GUI.req(script_path .. "ReaTrak_Classes/Class - Menubox.lua")()
GUI.req(script_path .. "ReaTrak_Classes/Class - Textbox.lua")()
GUI.req(script_path .. "ReaTrak_Classes/Class - Frame.lua")()
GUI.req(script_path .. "ReaTrak_Classes/Class - Options.lua")()
GUI.req(script_path .. "ReaTrak_Classes/Class - Window.lua")()

--------------------------------
 --REATRAK LOCAL GUI LIBRARY end
--------------------------------

--[[
--------------------------------
 --LOKASENNA GUI LIBRARY start
--------------------------------

-- NoIndex: true

--[[
  Lokasenna_GUI example

  - Getting user input before running an action; i.e. replacing GetUserInputs

]]--

--[[

-- The Core library must be loaded prior to anything else
reaper.MB("Using Lokasenna_GUI library", "Notice", 0)
    
local lib_path = reaper.GetExtState("Lokasenna_GUI", "lib_path_v2")
if not lib_path or lib_path == "" then
    reaper.MB("Couldn't load the Lokasenna_GUI library. Please run 'Script: Set Lokasenna_GUI v2 library path.lua' in your Action List.", "Whoops!", 0)
    return
end
loadfile(lib_path .. "Core.lua")()

GUI.req("Classes/Class - Slider.lua")()
GUI.req("Classes/Class - Button.lua")()
GUI.req("Classes/Class - Menubox.lua")()
GUI.req("Classes/Class - Options.lua")()
GUI.req("Classes/Class - Label.lua")()
GUI.req("Classes/Class - Frame.lua")()
GUI.req("Classes/Class - Textbox.lua")()

--------------------------------
 --LOKASENNA GUI LIBRARY end
--------------------------------
--]]

-- If any of the requested libraries weren't found, abort the script nicely.
if missing_lib then return 0 end


-- Button Colors  = {r, g, b, transparency}

GUI.colors["count_in"] = {255, 124, 192, 255}
GUI.colors["intro_post_fill"] = {119, 17, 174, 255}
GUI.colors["intro"] = {159, 22, 232, 255}
GUI.colors["intro_fill"] = {183, 128, 237, 255}
GUI.colors["verse_post_fill"] = {17, 39, 174, 255}
GUI.colors["verse"] = {55, 118, 235, 255}
GUI.colors["verse_fill"] = {113, 190, 241, 255}
GUI.colors["verse_ending"] = {151, 208, 245, 255}
GUI.colors["bridge_post_fill"] = {206, 105, 20, 255}
GUI.colors["bridge"] = {234, 133, 48, 255}
GUI.colors["bridge_fill"] = {239, 164, 100, 255}
GUI.colors["pre_chorus_post_fill"] = {206, 179, 20, 255}
GUI.colors["pre_chorus"] = {234, 208, 48, 255}
GUI.colors["pre_chorus_fill"] = {244, 231, 151, 255}
GUI.colors["chorus_post_fill"] = {11, 116, 39, 255}
GUI.colors["chorus"] = {17, 174, 59, 255}
GUI.colors["chorus_fill"] = {80, 237, 123, 255}
GUI.colors["chorus_ending"] = {158, 254, 182, 255}
GUI.colors["play_anywhere"] = {228, 26, 39, 250}
GUI.colors["drum_riff"] = {115, 23, 17, 255}
GUI.colors["hold"] = {191, 191, 191, 255}
GUI.colors["shot"] = {127, 127, 127, 255}
GUI.colors["rest"] = {0, 0, 0, 255}

--Text Colors

GUI.colors["btn_txt1"] = {255, 255, 255, 255} --white
GUI.colors["btn_txt2"] = {155, 155, 155, 255} --gray
GUI.colors["btn_txt3"] = {0, 0, 0, 255} --black

------------------------------------
-------- Functions  ----------------
------------------------------------


local function btn_click()
	
	-- Grab all of the user's settings into local variables,
	-- just to make it less awkward to work with
	local mode, thresh = GUI.Val("mnu_mode"), GUI.Val("sldr_thresh")
	local opts = GUI.Val("chk_opts")
	local time_sel, sel_track, glue = opts[1], opts[2], opts[3]
	
	-- Be nice, give the user an Undo point
	reaper.Undo_BeginBlock()
	
	reaper.ShowMessageBox(
		"This is where we pretend to perform some sort of fancy operation with the user's settings.\n\n"
		.."Working in "..tostring(GUI.elms.mnu_mode.optarray[mode])
		.." mode with a threshold of "..tostring(thresh).."db.\n\n"
		.."Apply only to time selection: "..tostring(time_sel).."\n"
		.."Apply only to selected track: "..tostring(sel_track).."\n"
		.."Glue the processed items together afterward: "..tostring(glue)
	, "Yay!", 0)
	
	
	reaper.Undo_EndBlock("Typical script options", 0)     
	
	-- Exit the script on the next update
	GUI.quit = true
	
end


 
 
function GetFilename(path)   
    local start, finish = path:find('[%w%s!-={-|]+[_%.].+')   
    return path:sub(start,#path) 
end 

function GetFileExtension(file_name)
  return file_name:match("^.+(%..+)$")
end

function GetFileName(url)
  return url:match("^.+/(.+)$")
end

function export(f, variable)
  f:write(variable)
  f:write("\n")
end

function rgbToHex(r, g, b)
    return string.format("#%0.2X%0.2X%0.2X", r, g, b)
end

function table.val_to_str ( v )
  if "string" == type( v ) then
    v = string.gsub( v, "\n", "\\n" )
    if string.match( string.gsub(v,"[^'\"]",""), '^"+$' ) then
	 return "'" .. v .. "'"
    end
    return '"' .. string.gsub(v,'"', '\\"' ) .. '"'
  else
    return "table" == type( v ) and table.tostring( v ) or
	 tostring( v )
  end
end

function table.key_to_str ( k )
  if "string" == type( k ) and string.match( k, "^[_%a][_%a%d]*$" ) then
    return k
  else
    return "[" .. table.val_to_str( k ) .. "]"
  end
end

function table.tostring( tbl )
  local result, done = {}, {}
  for k, v in ipairs( tbl ) do
    table.insert( result, table.val_to_str( v ) )
    done[ k ] = true
  end
  for k, v in pairs( tbl ) do
    if not done[ k ] then
	 table.insert( result,
	   table.key_to_str( k ) .. "=" .. table.val_to_str( v ) )
    end
  end
  return "{" .. table.concat( result, "," ) .. "}"
end

function time_to_region(beat_time)
   
   markeridx, regionidx = reaper.GetLastMarkerAndCurRegion( 0, beat_time )
   retval, isrgn, pos, rgnend, beat_time_name, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, regionidx )
   --current_name = beat_time_name
   return beat_time_name
end



function round(x)
  return x>=0 and math.floor(x+0.5) or math.ceil(x-0.5)
end

function last_bar()

   export(f, "| " .. b1b1 .. " | ")
end


local function btn_click_new_region()
    retval_measures, insert_measures_retvals_csv = reaper.GetUserInputs( "Insert Measures", 1, "Number of Measures", "" )
    
    
    time = reaper.GetCursorPosition()
    
    reaper.Main_OnCommand(40625,0) -- Time selection: Set start point
	  if retval_measures == true then
		for i = 0, insert_measures_retvals_csv - 1 do
		    reaper.Main_OnCommand(41042,0) -- Move edit cursor forward one measure
		end
	  
	  reaper.Main_OnCommand(40626,0) -- Time selection: Set end point
	  
	  reaper.Main_OnCommand(40174,0) -- Markers: Insert region from time selection
	  
	  reaper.Main_OnCommand(40635,0)  -- Time selection: Remove time selection 
	  
	  --commandID03 = reaper.NamedCommandLookup("_RS0b4230c08f384a4318c16aac4e2404c9fef86887")
	  --reaper.Main_OnCommand(commandID03, 0)  -- Script: ReaTrak Snap all regions to grid.eel
			    
	  reaper.SetEditCurPos(time, 1, 0)
	  
	  end
    end

local function btn_click_edit_chord()
    reaper.Main_OnCommand(40616, 0) -- Markers: Edit region near cursor
    end

local function btn_click_chord_split()
    retval, num_markers, num_regionsOut = reaper.CountProjectMarkers(0)
    
    desired_region_id = num_regionsOut
    
    time = reaper.GetCursorPosition()
    
    markeridx, regionidx = reaper.GetLastMarkerAndCurRegion(0, time)
    
    retval, isrgn, pos, rgnend, name, markrgnindexnumber, color = reaper.EnumProjectMarkers3(0, regionidx)
    
    if time == pos then goto finish end
    
    if time < rgnend then
	  
	  reaper.SetProjectMarker3( 0, markrgnindexnumber, 1, pos, time, name, color )
    
	  reaper.AddProjectMarker2(0, 1, time, rgnend, name, desired_region_id, color)
    end
    
    ::finish::  
    end


local function btn_click_chord_pre()
    edit_pos = reaper.GetCursorPosition()
    
    play = reaper.GetPlayState()
    if play > 0 then
	    pos = reaper.GetPlayPosition()
    else
	    pos = edit_pos
    end
    
    count_markers_regions, count_markersOut, count_regionsOut = reaper.CountProjectMarkers(0)
    
    i=1
    repeat
	    iRetval, bIsrgnOut, iPosOut, iRgnendOut, sNameOut, iMarkrgnindexnumberOut, iColorOur = reaper.EnumProjectMarkers3(0,count_markers_regions-i)
	    if iRetval >= 1 then
		    if bIsrgnOut == true and iPosOut < pos then
			    -- ACTION ON REGIONS HERE
			    reaper.SetEditCurPos(iPosOut,true,true) -- moveview and seekplay
			    break
		    end
		    i = i+1
	    end
    until iRetval == 0
	    
   
    end
 
local function btn_click_bar_pre()
    reaper.Main_OnCommand(41043, 0) -- Move edit cursor back one measure
    time = reaper.GetCursorPosition()
    grid_pos = reaper.SnapToGrid(0, time)
    reaper.SetEditCurPos2(0, grid_pos, false, false)
    end

local function btn_click_beat_pre()
    reaper.Main_OnCommand(41045, 0) -- Move edit cursor back one beat
    time = reaper.GetCursorPosition()
    grid_pos = reaper.SnapToGrid(0, time)
    reaper.SetEditCurPos2(0, grid_pos, false, false)
    end
  
local function btn_click_chord_next()
    edit_pos = reaper.GetCursorPosition()
    
    play = reaper.GetPlayState()
    if play > 0 then
	    pos = reaper.GetPlayPosition()
    else
	    pos = edit_pos
    end
    
    i=0
    repeat
	    iRetval, bIsrgnOut, iPosOut, iRgnendOut, sNameOut, iMarkrgnindexnumberOut, iColorOur = reaper.EnumProjectMarkers3(0,i)
	    if iRetval >= 1 then
		    if bIsrgnOut == true and iPosOut > pos then
			    -- ACTION ON REGIONS HERE
			    reaper.SetEditCurPos(iPosOut,true,true) -- moveview and seekplay
			    break
		    end
		    i = i+1
	    end
    until iRetval == 0
	    
    
    end

local function btn_click_bar_next()
    --reaper.Main_OnCommand(41040, 0) -- Move edit cursor to start of next measure
    reaper.Main_OnCommand(41042, 0) -- Move edit cursor forward one measure
    time = reaper.GetCursorPosition()
    grid_pos = reaper.SnapToGrid(0, time)
    reaper.SetEditCurPos2(0, grid_pos, false, false)    
    end

local function btn_click_beat_next()
    reaper.Main_OnCommand(41044, 0) -- Move edit cursor forward one beat
    time = reaper.GetCursorPosition()
    grid_pos = reaper.SnapToGrid(0, time)
    reaper.SetEditCurPos2(0, grid_pos, false, false)    
    end

function btn_click_chord()
    chord_name = GUI.Val("my_chord")
    retval, num_markers, num_regionsOut = reaper.CountProjectMarkers(0)
    
    desired_region_id = num_regionsOut
    
    time = reaper.GetCursorPosition()
    
    markeridx, regionidx = reaper.GetLastMarkerAndCurRegion(0, time)
    
    retval, isrgn, pos, rgnend, name, markrgnindexnumber, color = reaper.EnumProjectMarkers3(0, regionidx)
    
    if time == pos then goto finish end
    
    if time < rgnend then
	  
	  reaper.SetProjectMarker3( 0, markrgnindexnumber, 1, pos, time, name, color )
    
	  reaper.AddProjectMarker2(0, 1, time, rgnend, name, desired_region_id, color)
	  
	  goto finish 
    end
    
    ::finish::
    
    
	time = reaper.GetCursorPosition()
	markrgnindexnumber, rgnendOut = reaper.GetLastMarkerAndCurRegion(0, time)
	retval, isrgnOut, posOut, rgnendOut, nameOut, markrgnindexnumberOut = reaper.EnumProjectMarkers(rgnendOut)
	clear_name_flag = 0 --this flag is needed in the SetProjectMarker4 function to enable the region name to be cleared
    
	newName = chord_name
	
	reaper.SetProjectMarker4(0, markrgnindexnumberOut, true, posOut, rgnendOut, newName, 0, clear_name_flag)
	chord_type = ""
	btn_click_chord_next()
    end

function btn_click_verse_post_fill()
    time =  reaper.GetCursorPosition()
    markeridx, regionidx = reaper.GetLastMarkerAndCurRegion( 0, time )
    retval, isrgn, pos, rgnend, name_str, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, regionidx )
    reaper.SetProjectMarker4( 0, markrgnindexnumber, isrgn, pos, rgnend, name_str,(reaper.ColorToNative(17,39,174)|0x1000000), 0)
    btn_click_chord_next()
    end

function btn_click_verse_ending()
    time =  reaper.GetCursorPosition()
    markeridx, regionidx = reaper.GetLastMarkerAndCurRegion( 0, time )
    retval, isrgn, pos, rgnend, name_str, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, regionidx )
    reaper.SetProjectMarker4( 0, markrgnindexnumber, isrgn, pos, rgnend, name_str,(reaper.ColorToNative(151,208,254)|0x1000000), 0)
    btn_click_chord_next()
    end

function btn_click_chorus_post_fill()
    time =  reaper.GetCursorPosition()
    markeridx, regionidx = reaper.GetLastMarkerAndCurRegion( 0, time )
    retval, isrgn, pos, rgnend, name_str, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, regionidx )
    reaper.SetProjectMarker4( 0, markrgnindexnumber, isrgn, pos, rgnend, name_str,(reaper.ColorToNative(11,116,39)|0x1000000), 0)
    btn_click_chord_next()
    end
    
function btn_click_chorus_ending()
    time =  reaper.GetCursorPosition()
    markeridx, regionidx = reaper.GetLastMarkerAndCurRegion( 0, time )
    retval, isrgn, pos, rgnend, name_str, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, regionidx )
    reaper.SetProjectMarker4( 0, markrgnindexnumber, isrgn, pos, rgnend, name_str,(reaper.ColorToNative(158,245,182)|0x1000000), 0)
    btn_click_chord_next()
    end    
    
function btn_click_count_in()
    time =  reaper.GetCursorPosition()
    markeridx, regionidx = reaper.GetLastMarkerAndCurRegion( 0, time )
    retval, isrgn, pos, rgnend, name_str, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, regionidx )
    reaper.SetProjectMarker4( 0, markrgnindexnumber, isrgn, pos, rgnend, name_str,(reaper.ColorToNative(255,128,192)|0x1000000), 0)
    btn_click_chord_next()

    end    



function Main()

  

  retval_inputs, key = reaper.GetUserInputs( "Set Song Key", 1, "Key", "C" )
  

  master_tempo = reaper.Master_GetTempo()

	f = io.open(file, "w")
  
  export(f, "Name: ".. name_input)
  export(f, "Meter: 4/4")
  export(f, "Style: Swing")
  export(f, "Key: "..key)
  export(f, "Tempo: ".. round(master_tempo))
 
--[[
  Name: Guitar, 12-String Acoustic, Strumming Ev 120 demo
  Meter: 4/4
  Style: Swing
  Key: D
  Tempo: 120
  Chorus: bars 1 to 96 (repeat 1 times)
  Sections: 1A 9B 17A 25B 33A 41B 49A 57B 65A 73B 81A 89B 97A
--]] 

  retval, isrgn, pos, rgnend, name, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, 0 )
	 
  retval, measures, cml, fullbeats, cdenom = reaper.TimeMap2_timeToBeats( 0, pos )
  
  --Msg("start ".. measures)
  
  
  retval, num_markers, num_regions = reaper.CountProjectMarkers( 0 )
  --Msg("num_regions ".. num_regions)
  for i = 0, num_regions -1 do
	retval, isrgn, pos, rgnend, name, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, i )
	if color == reaper.ColorToNative(17,39,174)|0x1000000 or color == reaper.ColorToNative(11,116,39)|0x1000000 then
	   retval, measures, cml, fullbeats, cdenom = reaper.TimeMap2_timeToBeats( 0, pos )
	   start_measure = measures break 
	end
  end
  
  retval, isrgn, pos, rgnend, name, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, num_regions -2 )
	 
  retval, measures, cml, fullbeats, cdenom = reaper.TimeMap2_timeToBeats( 0, pos )
  
  end_bar = measures
  
  export(f, "Chorus: bars ".. 1 .." to ".. measures-(start_measure -1) .. " (repeat 1 times)")
  
  part_marker_count = 0
  
  offset = 0
  
  first_bar = 0
  
  section_table = {}
  
  for i = 0, num_regions -1 do
    retval, isrgn, pos, rgnend, name, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, i )
    
    --Msg(i .." ".. name)
    
    retval, measures, cml, fullbeats, cdenom = reaper.TimeMap2_timeToBeats( 0, pos )
    --Msg(name .." ".. measures .." ".. cml .." ".. fullbeats .." ".. cdenom .." ".. (fullbeats/cml)-measures )
    --if color == reaper.ColorToNative(255,128,192)|0x1000000 then offset = 2 Msg("offset ".. offset)end
    --reaper.ColorFromNative(iColorOur)
    if color == reaper.ColorToNative(17,39,174)|0x1000000 or color == reaper.ColorToNative(11,116,39)|0x1000000 then
	  
	  part_marker_count = part_marker_count +1
	  
	  if first_bar == 0 then 
	  
		retval, measures, cml, fullbeats, cdenom = reaper.TimeMap2_timeToBeats( 0, pos )
		
		first_bar = measures
		
		first_bar_region = i
		
		first_bar_time = reaper.TimeMap2_beatsToTime( 0, fullbeats )
		
		--Msg("first_bar_time ".. first_bar_time .." pos ".. pos)
		
		retval, qn_start, qn_end, timesig_num, timesig_denom, tempo = reaper.TimeMap_GetMeasureInfo( 0, measures )
	  
		--Msg("qn_start ".. qn_start)
	  end
	  
	  if color == reaper.ColorToNative(17,39,174)|0x1000000 then
		part_marker = "A" 
		table.insert(section_table,((measures)-start_measure+1) .. part_marker)
	  end 
	  
	  if color == reaper.ColorToNative(11,116,39)|0x1000000 then
		part_marker = "B"
		table.insert(section_table,((measures)-start_measure+1) .. part_marker)
	  end 
	  
    end
    
    if color == reaper.ColorToNative(151,208,254)|0x1000000 or color == reaper.ColorToNative(158,245,182)|0x1000000 then   
	  
	  if color == reaper.ColorToNative(151,208,254)|0x1000000 then
		part_marker = "A" 
		table.insert(section_table,((measures)-start_measure+1) .. part_marker)
	  end 
	  
	  if color == reaper.ColorToNative(158,245,182)|0x1000000 then
		part_marker = "B"
		table.insert(section_table,((measures)-start_measure+1) .. part_marker)
	  end       
		   
    end     
	  
    
	  
	  
	  --Msg(part_marker)
	  --Msg(measures+1)
	  
    
  end

		   
    section_markers = table.tostring( section_table )
    section_markers = string.gsub(section_markers, [[","]], " ")
    section_markers = string.gsub(section_markers, [["]], " ")
    section_markers = string.gsub(section_markers, [[{]], "")
    section_markers = string.gsub(section_markers, [[}]], "")
    export(f, "Sections:".. section_markers)
  --Msg(end_bar+1)
  final_time = end_bar+1
  end_time = reaper.TimeMap2_beatsToTime( 0, 3, final_time ) 
  --Msg("end_time ".. end_time)
  
 
  
  for i = first_bar, end_bar -1 do
    --markeridx, regionidx = reaper.GetLastMarkerAndCurRegion( 0, beat_time )
    --retval, isrgn, pos, rgnend, beat_time_name, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, regionidx )
  --Msg("current_name "..current_name) 
    if i == first_bar then
	  beat_time = reaper.TimeMap2_beatsToTime( 0, 0, i )
	  markeridx, regionidx = reaper.GetLastMarkerAndCurRegion( 0, beat_time )
	  retval, isrgn, pos, rgnend, name, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, regionidx )
	  first_bar_region = regionidx       
    else 
	  beat_time = reaper.TimeMap2_beatsToTime( 0, 0, first_bar )
    end
    --Msg("current_name "..current_name) 
    retval, measures, cml, fullbeats, cdenom = reaper.TimeMap2_timeToBeats( 0, beat_time )  
  
    --end_time = reaper.TimeMap2_beatsToTime( 0, 3, end_bar )
    
    bar_1_beat_1 = reaper.TimeMap2_beatsToTime( 0,0, measures )
    b1b1 = time_to_region(bar_1_beat_1) current = b1b1
    if bar_1_beat_1 > end_time then last_bar() break end
    bar_1_beat_2 = reaper.TimeMap2_beatsToTime( 0, 1, measures )
    b1b2 = time_to_region(bar_1_beat_2) 
    if b1b2 == current then b1b2 = "_" end
    current = b1b2
    if bar_1_beat_2 > end_time then last_bar() break end
    bar_1_beat_3 = reaper.TimeMap2_beatsToTime( 0, 2, measures )
    b1b3 = time_to_region(bar_1_beat_3) 
    if b1b3 == current then b1b3 = "_" end
    current = b1b3
    if bar_1_beat_3 > end_time then last_bar() break end
    bar_1_beat_4 = reaper.TimeMap2_beatsToTime( 0, 3, measures )
    b1b4 = time_to_region(bar_1_beat_4) 
    if b1b4 == current then b1b4 = "_" end
    current = b1b4 
    if bar_1_beat_4 > end_time then last_bar() break end

    bar_2_beat_1 = reaper.TimeMap2_beatsToTime( 0, 0, measures+1 )
    b2b1 = time_to_region(bar_2_beat_1) current = b2b1
    if bar_2_beat_1 > end_time then last_bar() break end
    bar_2_beat_2 = reaper.TimeMap2_beatsToTime( 0, 1, measures+1 )
    b2b2 = time_to_region(bar_2_beat_2) 
    if b2b2 == current then b2b2 = "_" end
    current = b2b2
    if bar_2_beat_2 > end_time then last_bar() break end
    bar_2_beat_3 = reaper.TimeMap2_beatsToTime( 0, 2, measures+1 )
    b2b3 = time_to_region(bar_2_beat_3) 
    if b2b3 == current then b2b3 = "_" end
    current = b2b3
    if bar_2_beat_3 > end_time then last_bar() break end
    bar_2_beat_4 = reaper.TimeMap2_beatsToTime( 0, 3, measures+1 )
    b2b4 = time_to_region(bar_2_beat_4) 
    if b2b4 == current then b2b4 = "_" end
    current = b2b4              
    if bar_2_beat_4 > end_time then last_bar() break end
    
    bar_3_beat_1 = reaper.TimeMap2_beatsToTime( 0, 0, measures+2 )
    b3b1 = time_to_region(bar_3_beat_1) current = b3b1
    if bar_3_beat_1 > end_time then last_bar() break end
    bar_3_beat_2 = reaper.TimeMap2_beatsToTime( 0, 1, measures+2 )
    b3b2 = time_to_region(bar_3_beat_2) 
    if b3b2 == current then b3b2 = "_" end
    current = b3b2
    if bar_3_beat_2 > end_time then last_bar() break end
    bar_3_beat_3 = reaper.TimeMap2_beatsToTime( 0, 2, measures+2 )
    b3b3 = time_to_region(bar_3_beat_3) 
    if b3b3 == current then b3b3 = "_" end
    current = b3b3
    if bar_3_beat_3 > end_time then last_bar() break end
    bar_3_beat_4 = reaper.TimeMap2_beatsToTime( 0, 3, measures+2 )
    b3b4 = time_to_region(bar_3_beat_4) 
    if b3b4 == current then b3b4 = "_" end
    current = b3b4
    if bar_3_beat_4 > end_time then last_bar() break end

    bar_4_beat_1 = reaper.TimeMap2_beatsToTime( 0, 0, measures+3 )
    b4b1 = time_to_region(bar_4_beat_1) current = b4b1
    if bar_4_beat_1 > end_time then last_bar() break end
    bar_4_beat_2 = reaper.TimeMap2_beatsToTime( 0, 1, measures+3 )
    b4b2 = time_to_region(bar_4_beat_2) 
    if b4b2 == current then b4b2 = "_" end
    current = b4b2
    if bar_4_beat_2 > end_time then last_bar() break end
    bar_4_beat_3 = reaper.TimeMap2_beatsToTime( 0, 2, measures+3 )
    b4b3 = time_to_region(bar_4_beat_3) 
    if b4b3 == current then b4b3 = "_" end
    current = b4b3
    if bar_4_beat_3 > end_time then last_bar() break end
    bar_4_beat_4 = reaper.TimeMap2_beatsToTime( 0, 3, measures+3 )
    b4b4 = time_to_region(bar_4_beat_4) 
    if b4b4 == current then b4b4 = "_" end
    current = b4b4
    if bar_4_beat_4 > end_time then last_bar() break end
    --Msg("bar_4_beat_4 ".. bar_4_beat_4 .." end_time ".. end_time)
    
    line = 
		"| " .. b1b1 .. " " .. b1b2 .. " " .. b1b3 .. " " .. b1b4 .. " | " .. b2b1 .. " " .. b2b2 .. " " .. b2b3 .. " " .. b2b4 .. " | " .. b3b1 .. " " .. b3b2 .. " " .. b3b3 .. " " .. b3b4 .. " | " .. b4b1 .. " " .. b4b2 .. " " .. b4b3 .. " " .. b4b4 .. " |"
    export(f, line)
    
    first_bar = first_bar +4
    
    --if first_bar => end_bar then break end
    
  end  
  
 

  -- CLOSE FILE
  f:close() -- never forget to close the file
  
  local info = debug.getinfo(1,'S');
  script_path = info.source:match[[^@?(.*[\/])[^\/]-$]]
 
  --file
  
  --filename_no_ext = string.gsub(file, '[.]%w%w%w$','')
  
  mjb_mid_file = file .. ".mid"
  
  --Msg(mjb_mid_file)
  
  file2 = tostring([["]].. file ..[["]])
  
  --Msg(file2)
  
  mjb2lite_path = script_path .. "mjb2lite\\"
  
  --Msg(mjb2lite_path)
  
  reaper.ExecProcess([[cmd.exe /C " cd ]] .. mjb2lite_path .. [[& mjb2lite.exe ]].. file2 ..[["]],0)
  --os.execute([[cmd.exe /C " cd ]] .. script_path .. [[ mjb2lite/ & "mjb2lite.exe" ]].. file2 ..[["]])
  --reaper.ExecProcess([[cmd.exe /C " cd ]] .. script_path .. [[sonic/win32/ & set VAMP_PATH=]] .. script_path .. [[sonic/win32/vamp; ]].. script_path .. [[ & "sonic-annotator.exe" -d vamp:nnls-chroma:chordino:simplechord -w csv "]].. filename ..[["]],0)
  
  
  reaper.InsertMedia( mjb_mid_file, 1 )

end 

function btn_click_create_file()

   commandID30 = reaper.NamedCommandLookup("_SWSMARKERLIST8")
   reaper.Main_OnCommand(commandID30, 0) -- SWS: Renumber region IDs
   
   reaper.Main_OnCommand(41761,0)  -- Regions: Go to region 01 after current region finishes playing (smooth seek)


   ::start_dialog::
   
   if not reaper.JS_Dialog_BrowseForSaveFile then
	Msg("Please install JS_ReaScript REAPER extension, available in Reapack extension, under ReaTeam Extensions repository.")
   else
   
    --retval, file = reaper.JS_Dialog_BrowseForSaveFile( "Save Chords as *.mjb", '.mjb', ".mjb", 'mjb files (.mjb)\0*.mjb\0All Files (*.*)\0*.*\0' )
    --reaper.JS_Dialog_BrowseForSaveFile("Save As Reabank", initialFolder, initialFile, extensionList)
    retval, file = reaper.JS_Dialog_BrowseForSaveFile("Save Chords as *.mjb", ".mjb", "", 'mjb files (.mjb)\0*.mjb\0All Files (*.*)\0*.*\0')
    
    if retval and file == '' then
	  goto finish
    end
    
    if retval and reaper.file_exists( file ..".mjb" ) then
	  reaper.MB("File Exist","File Exist",0)
	  goto start_dialog
    end   
    
    if retval and file ~= '' then
    
	 if GetFileExtension(file) ~= ".mjb" then
	    file = (file.. ".mjb")
	 end  
	 file_path = GetFilename(file)
    
	 name_input = string.gsub(file_path, '[.]%w%w%w$','')
	 --Msg("name_input ".. name_input)
	 reaper.defer(Main)
    end
    
   ::finish::
   --if not retval then 
	
   
    --file = (file.. ".mjb")
    
	
   end


end

------------------------------------
-------- Window settings -----------
------------------------------------


GUI.name = "MYJAZZBAND 2 LITE by Andrew Choi for 4/4 Swing Jazz Track"
GUI.x, GUI.y, GUI.w, GUI.h = 0, 0, 400, 400
GUI.anchor, GUI.corner = "mouse", "C"




------------------------------------
-------- GUI Elements --------------
------------------------------------


--[[     

	Button          z,      x,      y,      w,      h, caption, func[, ...]
	Checklist     z,      x,      y,      w,      h, caption, opts[, dir, pad]
	Menubox          z,      x,      y,      w,      h, caption, opts, pad, noarrow]
	Slider          z,      x,      y,      w,      caption, min, max, defaults[, inc, dir]
	
]]--

--GUI.New("mnu_mode",     "Menubox",          1, 64,     32,  72, 20, "Mode:", "Auto,Punch,Step")
--GUI.New("chk_opts",     "Checklist",     1, 192,     32,  192, 96, "Options", "Only in time selection,Only on selected track,Glue items when finished", "v", 4)
--GUI.New("sldr_thresh", "Slider",     1, 32,  96, 128, "Threshold", -60, 0, 48, nil, "h")
--GUI.New("btn_go",     "Button",          1, 168, 152, 64, 24, "Go!", btn_click)

GUI.New("new_region_btn",      "Button",           1, 20, 40, 125, 20, "New Region at Cursor", btn_click_new_region)
GUI.elms.new_region_btn.col_txt = "yellow"
GUI.elms.new_region_btn.col_fill = "shot"

GUI.New("edit_chord2_btn",      "Button",           1, 165, 40, 70, 20, "Edit Chord", btn_click_edit_chord)
GUI.elms.edit_chord2_btn.col_txt = "yellow"
GUI.elms.edit_chord2_btn.col_fill = "shot"

GUI.New("chord_split_btn",      "Button",           1, 250, 40, 125, 20, "Split Chord at Cursor", btn_click_chord_split)
GUI.elms.chord_split_btn.col_txt = "yellow"
GUI.elms.chord_split_btn.col_fill = "shot"

GUI.New("chord_pre_btn",      "Button",           1, 80, 80, 115, 20, "< Previous Chord", btn_click_chord_pre)
GUI.elms.chord_pre_btn.col_txt = "yellow"
GUI.elms.chord_pre_btn.col_fill = "shot"

GUI.New("chord_next_btn",      "Button",           1, 200, 80, 95, 20, "Next Chord >", btn_click_chord_next)
GUI.elms.chord_next_btn.col_txt = "yellow"
GUI.elms.chord_next_btn.col_fill = "shot"

GUI.New("bar_pre_btn",      "Button",           1, 60, 120, 50, 20, "< Bar", btn_click_bar_pre)
GUI.elms.bar_pre_btn.col_txt = "yellow"
GUI.elms.bar_pre_btn.col_fill = "shot"

GUI.New("bar_next_btn",      "Button",          1, 260, 120, 50, 20, "Bar >", btn_click_bar_next)
GUI.elms.bar_next_btn.col_txt = "yellow"
GUI.elms.bar_next_btn.col_fill = "shot"

GUI.New("beat_pre_btn",      "Button",           1, 130, 120, 45, 20, "< Beat", btn_click_beat_pre)
GUI.elms.beat_pre_btn.col_txt = "yellow"
GUI.elms.beat_pre_btn.col_fill = "shot"

GUI.New("beat_next_btn",      "Button",           1, 195, 120, 45, 20, "Beat >", btn_click_beat_next)
GUI.elms.beat_next_btn.col_txt = "yellow"
GUI.elms.beat_next_btn.col_fill = "shot"

GUI.New("my_chord",   "Textbox",     1, 120, 160, 120, 20, "", 4)

GUI.New("btn_chord",     "Button",          1, 20, 160, 90, 20, "Insert Chord >", btn_click_chord)

GUI.New("verse_post_fill_btn",      "Button",           1, 20, 200, 95, 20, "Verse Post Fill", btn_click_verse_post_fill)
GUI.elms.verse_post_fill_btn.col_txt = "btn_txt1"
GUI.elms.verse_post_fill_btn.col_fill = "verse_post_fill" 

GUI.New("verse_ending_btn",      "Button",           1, 140, 200, 90, 20, "Verse Ending ", btn_click_verse_ending)
GUI.elms.verse_ending_btn.col_txt = "btn_txt3"
GUI.elms.verse_ending_btn.col_fill = "verse_ending"

GUI.New("chorus_post_fill_btn",      "Button",           1, 20, 240, 95, 20, "Chorus Post Fill", btn_click_chorus_post_fill)
GUI.elms.chorus_post_fill_btn.col_txt = "btn_txt1"
GUI.elms.chorus_post_fill_btn.col_fill = "chorus_post_fill"

GUI.New("chorus_ending_btn",      "Button",           1, 140, 240, 90, 20, "Chorus Ending ", btn_click_chorus_ending)
GUI.elms.chorus_ending_btn.col_txt = "btn_txt3"
GUI.elms.chorus_ending_btn.col_fill = "chorus_ending"

GUI.New("count_in_btn",      "Button",           1, 20, 280, 95, 20, "2 Bar Count-In", btn_click_count_in)
GUI.elms.count_in_btn.col_txt = "btn_txt3"
GUI.elms.count_in_btn.col_fill = "count_in" 

GUI.New("btn_create_file",     "Button",          1, 250, 200, 130, 44, "Create Jazz Midi Track", btn_click_create_file)

--GUI.New("lb_Sect",      "Label",           3, 100, 280, "ReaTrak Sections", true, 1)

GUI.New("my_frm",      "Frame",           1,20, 310, 340, 65, true, true, "elm_bg", 4)
GUI.elms.my_frm.text = "*Create a 2 bar count-in *Mark the verse/chorus with 1 bar *Add verse/chorus fill where you like by splitting the region chord to a 1 bar fill *Create an ending with a fill then a 1 bar ending color"
GUI.elms.my_frm.col_txt = "white"

GUI.Init()
GUI.Main()
