--[[
 * ReaTrak region chords to 4-4 jazz swing midi.lua
 * Uses MYJAZZBAND 2 LITE by Andrew Choi.
 * https://sixthhappiness.github.io/articles/myjazzband-2-lite/index.html

 * Thanks to X-Raym for:
 
 * ReaScript Name: Export markers and regions from tab-delimited CSV file
 * Description: See title.
 * Instructions: Select a track. Run.
 * Author: X-Raym
 * Author URI: http://www.extremraym.com
 * Repository: GitHub > X-Raym > EEL Scripts for Cockos REAPER
 * Repository URI: https://github.com/X-Raym/REAPER-EEL-Scripts
 * Links
    Forum Thread https://forum.cockos.com/showthread.php?p=1670961
 * Licence: GPL v3
 * REAPER: 5.0
 * Version: 1.0.3
--]]

--[[
 * Changelog:
 * v1.0.3 (2019-12-20)
  # Msg
 * v1.0.2 (2019-11-20)
  # Header tab
 * v1.0.1 (2019-01-26)
  # Extension
 * v1.0 (2019-01-26)
  + Initial Release
--]]

function Msg(val)
  reaper.ShowConsoleMsg(tostring(val).."\n")
end

-- ignor @ regions

 
 
function GetFilename(path)   
    local start, finish = path:find('[%w%s!-={-|]+[_%.].+')   
    return path:sub(start,#path) 
end 

function GetFileExtension(file_name)
  return file_name:match("^.+(%..+)$")
end

function GetFileName(url)
  return url:match("^.+/(.+)$")
end

function export(f, variable)
  f:write(variable)
  f:write("\n")
end

function rgbToHex(r, g, b)
    return string.format("#%0.2X%0.2X%0.2X", r, g, b)
end

function table.val_to_str ( v )
  if "string" == type( v ) then
    v = string.gsub( v, "\n", "\\n" )
    if string.match( string.gsub(v,"[^'\"]",""), '^"+$' ) then
      return "'" .. v .. "'"
    end
    return '"' .. string.gsub(v,'"', '\\"' ) .. '"'
  else
    return "table" == type( v ) and table.tostring( v ) or
      tostring( v )
  end
end

function table.key_to_str ( k )
  if "string" == type( k ) and string.match( k, "^[_%a][_%a%d]*$" ) then
    return k
  else
    return "[" .. table.val_to_str( k ) .. "]"
  end
end

function table.tostring( tbl )
  local result, done = {}, {}
  for k, v in ipairs( tbl ) do
    table.insert( result, table.val_to_str( v ) )
    done[ k ] = true
  end
  for k, v in pairs( tbl ) do
    if not done[ k ] then
      table.insert( result,
        table.key_to_str( k ) .. "=" .. table.val_to_str( v ) )
    end
  end
  return "{" .. table.concat( result, "," ) .. "}"
end

function time_to_region(beat_time)
   
   markeridx, regionidx = reaper.GetLastMarkerAndCurRegion( 0, beat_time )
   retval, isrgn, pos, rgnend, beat_time_name, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, regionidx )
   --current_name = beat_time_name
   return beat_time_name
end



function round(x)
  return x>=0 and math.floor(x+0.5) or math.ceil(x-0.5)
end

function last_bar()

   export(f, "| " .. b1b1 .. " | ")
end

function Main()

  

  retval_inputs, key = reaper.GetUserInputs( "Set Song Key", 1, "Key", "C" )
  

  master_tempo = reaper.Master_GetTempo()

     f = io.open(file, "w")
  
  export(f, "Name: ".. name_input)
  export(f, "Meter: 4/4")
  export(f, "Style: Swing")
  export(f, "Key: "..key)
  export(f, "Tempo: ".. round(master_tempo))
 
--[[
  Name: Guitar, 12-String Acoustic, Strumming Ev 120 demo
  Meter: 4/4
  Style: Swing
  Key: D
  Tempo: 120
  Chorus: bars 1 to 96 (repeat 1 times)
  Sections: 1A 9B 17A 25B 33A 41B 49A 57B 65A 73B 81A 89B 97A
--]] 

  retval, isrgn, pos, rgnend, name, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, 0 )
      
  retval, measures, cml, fullbeats, cdenom = reaper.TimeMap2_timeToBeats( 0, pos )
  
  --Msg("start ".. measures)
  
  
  retval, num_markers, num_regions = reaper.CountProjectMarkers( 0 )
  --Msg("num_regions ".. num_regions)
  for i = 0, num_regions -1 do
     retval, isrgn, pos, rgnend, name, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, i )
     if color == reaper.ColorToNative(17,39,174)|0x1000000 or color == reaper.ColorToNative(11,116,39)|0x1000000 then
        retval, measures, cml, fullbeats, cdenom = reaper.TimeMap2_timeToBeats( 0, pos )
        start_measure = measures break 
     end
  end
  
  retval, isrgn, pos, rgnend, name, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, num_regions -2 )
      
  retval, measures, cml, fullbeats, cdenom = reaper.TimeMap2_timeToBeats( 0, pos )
  
  end_bar = measures
  
  export(f, "Chorus: bars ".. 1 .." to ".. measures-(start_measure -1) .. " (repeat 1 times)")
  
  part_marker_count = 0
  
  offset = 0
  
  first_bar = 0
  
  section_table = {}
  
  for i = 0, num_regions -1 do
    retval, isrgn, pos, rgnend, name, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, i )
    
    --Msg(i .." ".. name)
    
    retval, measures, cml, fullbeats, cdenom = reaper.TimeMap2_timeToBeats( 0, pos )
    --Msg(name .." ".. measures .." ".. cml .." ".. fullbeats .." ".. cdenom .." ".. (fullbeats/cml)-measures )
    --if color == reaper.ColorToNative(255,128,192)|0x1000000 then offset = 2 Msg("offset ".. offset)end
    --reaper.ColorFromNative(iColorOur)
    if color == reaper.ColorToNative(17,39,174)|0x1000000 or color == reaper.ColorToNative(11,116,39)|0x1000000 then
       
       part_marker_count = part_marker_count +1
       
       if first_bar == 0 then 
       
          retval, measures, cml, fullbeats, cdenom = reaper.TimeMap2_timeToBeats( 0, pos )
          
          first_bar = measures
          
          first_bar_region = i
          
          first_bar_time = reaper.TimeMap2_beatsToTime( 0, fullbeats )
          
          --Msg("first_bar_time ".. first_bar_time .." pos ".. pos)
          
          retval, qn_start, qn_end, timesig_num, timesig_denom, tempo = reaper.TimeMap_GetMeasureInfo( 0, measures )
       
          --Msg("qn_start ".. qn_start)
       end
       
       if color == reaper.ColorToNative(17,39,174)|0x1000000 then
          part_marker = "A" 
          table.insert(section_table,((measures)-start_measure+1) .. part_marker)
       end 
       
       if color == reaper.ColorToNative(11,116,39)|0x1000000 then
          part_marker = "B"
          table.insert(section_table,((measures)-start_measure+1) .. part_marker)
       end 
       
    end
    
    if color == reaper.ColorToNative(151,208,254)|0x1000000 or color == reaper.ColorToNative(158,245,182)|0x1000000 then   
       
       if color == reaper.ColorToNative(151,208,254)|0x1000000 then
          part_marker = "A" 
          table.insert(section_table,((measures)-start_measure+1) .. part_marker)
       end 
       
       if color == reaper.ColorToNative(158,245,182)|0x1000000 then
          part_marker = "B"
          table.insert(section_table,((measures)-start_measure+1) .. part_marker)
       end       
             
    end     
       
    
       
       
       --Msg(part_marker)
       --Msg(measures+1)
       
    
  end

             
    section_markers = table.tostring( section_table )
    section_markers = string.gsub(section_markers, [[","]], " ")
    section_markers = string.gsub(section_markers, [["]], " ")
    section_markers = string.gsub(section_markers, [[{]], "")
    section_markers = string.gsub(section_markers, [[}]], "")
    export(f, "Sections:".. section_markers)
  --Msg(end_bar+1)
  final_time = end_bar+1
  end_time = reaper.TimeMap2_beatsToTime( 0, 3, final_time ) 
  --Msg("end_time ".. end_time)
  
 
  
  for i = first_bar, end_bar -1 do
    --markeridx, regionidx = reaper.GetLastMarkerAndCurRegion( 0, beat_time )
    --retval, isrgn, pos, rgnend, beat_time_name, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, regionidx )
  --Msg("current_name "..current_name) 
    if i == first_bar then
       beat_time = reaper.TimeMap2_beatsToTime( 0, 0, i )
       markeridx, regionidx = reaper.GetLastMarkerAndCurRegion( 0, beat_time )
       retval, isrgn, pos, rgnend, name, markrgnindexnumber, color = reaper.EnumProjectMarkers3( 0, regionidx )
       first_bar_region = regionidx       
    else 
       beat_time = reaper.TimeMap2_beatsToTime( 0, 0, first_bar )
    end
    --Msg("current_name "..current_name) 
    retval, measures, cml, fullbeats, cdenom = reaper.TimeMap2_timeToBeats( 0, beat_time )  
  
    --end_time = reaper.TimeMap2_beatsToTime( 0, 3, end_bar )
    
    bar_1_beat_1 = reaper.TimeMap2_beatsToTime( 0,0, measures )
    b1b1 = time_to_region(bar_1_beat_1) current = b1b1
    if bar_1_beat_1 > end_time then last_bar() break end
    bar_1_beat_2 = reaper.TimeMap2_beatsToTime( 0, 1, measures )
    b1b2 = time_to_region(bar_1_beat_2) 
    if b1b2 == current then b1b2 = "_" end
    current = b1b2
    if bar_1_beat_2 > end_time then last_bar() break end
    bar_1_beat_3 = reaper.TimeMap2_beatsToTime( 0, 2, measures )
    b1b3 = time_to_region(bar_1_beat_3) 
    if b1b3 == current then b1b3 = "_" end
    current = b1b3
    if bar_1_beat_3 > end_time then last_bar() break end
    bar_1_beat_4 = reaper.TimeMap2_beatsToTime( 0, 3, measures )
    b1b4 = time_to_region(bar_1_beat_4) 
    if b1b4 == current then b1b4 = "_" end
    current = b1b4 
    if bar_1_beat_4 > end_time then last_bar() break end

    bar_2_beat_1 = reaper.TimeMap2_beatsToTime( 0, 0, measures+1 )
    b2b1 = time_to_region(bar_2_beat_1) current = b2b1
    if bar_2_beat_1 > end_time then last_bar() break end
    bar_2_beat_2 = reaper.TimeMap2_beatsToTime( 0, 1, measures+1 )
    b2b2 = time_to_region(bar_2_beat_2) 
    if b2b2 == current then b2b2 = "_" end
    current = b2b2
    if bar_2_beat_2 > end_time then last_bar() break end
    bar_2_beat_3 = reaper.TimeMap2_beatsToTime( 0, 2, measures+1 )
    b2b3 = time_to_region(bar_2_beat_3) 
    if b2b3 == current then b2b3 = "_" end
    current = b2b3
    if bar_2_beat_3 > end_time then last_bar() break end
    bar_2_beat_4 = reaper.TimeMap2_beatsToTime( 0, 3, measures+1 )
    b2b4 = time_to_region(bar_2_beat_4) 
    if b2b4 == current then b2b4 = "_" end
    current = b2b4              
    if bar_2_beat_4 > end_time then last_bar() break end
    
    bar_3_beat_1 = reaper.TimeMap2_beatsToTime( 0, 0, measures+2 )
    b3b1 = time_to_region(bar_3_beat_1) current = b3b1
    if bar_3_beat_1 > end_time then last_bar() break end
    bar_3_beat_2 = reaper.TimeMap2_beatsToTime( 0, 1, measures+2 )
    b3b2 = time_to_region(bar_3_beat_2) 
    if b3b2 == current then b3b2 = "_" end
    current = b3b2
    if bar_3_beat_2 > end_time then last_bar() break end
    bar_3_beat_3 = reaper.TimeMap2_beatsToTime( 0, 2, measures+2 )
    b3b3 = time_to_region(bar_3_beat_3) 
    if b3b3 == current then b3b3 = "_" end
    current = b3b3
    if bar_3_beat_3 > end_time then last_bar() break end
    bar_3_beat_4 = reaper.TimeMap2_beatsToTime( 0, 3, measures+2 )
    b3b4 = time_to_region(bar_3_beat_4) 
    if b3b4 == current then b3b4 = "_" end
    current = b3b4
    if bar_3_beat_4 > end_time then last_bar() break end

    bar_4_beat_1 = reaper.TimeMap2_beatsToTime( 0, 0, measures+3 )
    b4b1 = time_to_region(bar_4_beat_1) current = b4b1
    if bar_4_beat_1 > end_time then last_bar() break end
    bar_4_beat_2 = reaper.TimeMap2_beatsToTime( 0, 1, measures+3 )
    b4b2 = time_to_region(bar_4_beat_2) 
    if b4b2 == current then b4b2 = "_" end
    current = b4b2
    if bar_4_beat_2 > end_time then last_bar() break end
    bar_4_beat_3 = reaper.TimeMap2_beatsToTime( 0, 2, measures+3 )
    b4b3 = time_to_region(bar_4_beat_3) 
    if b4b3 == current then b4b3 = "_" end
    current = b4b3
    if bar_4_beat_3 > end_time then last_bar() break end
    bar_4_beat_4 = reaper.TimeMap2_beatsToTime( 0, 3, measures+3 )
    b4b4 = time_to_region(bar_4_beat_4) 
    if b4b4 == current then b4b4 = "_" end
    current = b4b4
    if bar_4_beat_4 > end_time then last_bar() break end
    --Msg("bar_4_beat_4 ".. bar_4_beat_4 .." end_time ".. end_time)
    
    line = 
          "| " .. b1b1 .. " " .. b1b2 .. " " .. b1b3 .. " " .. b1b4 .. " | " .. b2b1 .. " " .. b2b2 .. " " .. b2b3 .. " " .. b2b4 .. " | " .. b3b1 .. " " .. b3b2 .. " " .. b3b3 .. " " .. b3b4 .. " | " .. b4b1 .. " " .. b4b2 .. " " .. b4b3 .. " " .. b4b4 .. " |"
    export(f, line)
    
    first_bar = first_bar +4
    
    --if first_bar => end_bar then break end
    
  end  
  
 

  -- CLOSE FILE
  f:close() -- never forget to close the file
  
  local info = debug.getinfo(1,'S');
  script_path = info.source:match[[^@?(.*[\/])[^\/]-$]]
 
  --file
  
  --filename_no_ext = string.gsub(file, '[.]%w%w%w$','')
  
  mjb_mid_file = file .. ".mid"
  
  --Msg(mjb_mid_file)
  
  file2 = tostring([["]].. file ..[["]])
  
  --Msg(file2)
  
  mjb2lite_path = script_path .. "mjb2lite\\"
  
  --Msg(mjb2lite_path)
  
  reaper.ExecProcess([[cmd.exe /C " cd ]] .. mjb2lite_path .. [[& mjb2lite.exe ]].. file2 ..[["]],0)
  --os.execute([[cmd.exe /C " cd ]] .. script_path .. [[ mjb2lite/ & "mjb2lite.exe" ]].. file2 ..[["]])
  --reaper.ExecProcess([[cmd.exe /C " cd ]] .. script_path .. [[sonic/win32/ & set VAMP_PATH=]] .. script_path .. [[sonic/win32/vamp; ]].. script_path .. [[ & "sonic-annotator.exe" -d vamp:nnls-chroma:chordino:simplechord -w csv "]].. filename ..[["]],0)
  
  
  reaper.InsertMedia( mjb_mid_file, 1 )

end 



::start_dialog::

if not reaper.JS_Dialog_BrowseForSaveFile then
  Msg("Please install JS_ReaScript REAPER extension, available in Reapack extension, under ReaTeam Extensions repository.")
else

 --retval, file = reaper.JS_Dialog_BrowseForSaveFile( "Save Chords as *.mjb", '.mjb', ".mjb", 'mjb files (.mjb)\0*.mjb\0All Files (*.*)\0*.*\0' )
 --reaper.JS_Dialog_BrowseForSaveFile("Save As Reabank", initialFolder, initialFile, extensionList)
 retval, file = reaper.JS_Dialog_BrowseForSaveFile("Save Chords as *.mjb", ".mjb", "", 'mjb files (.mjb)\0*.mjb\0All Files (*.*)\0*.*\0')
 
 if retval and file == '' then
    goto finish
 end
 
 if retval and reaper.file_exists( file ..".mjb" ) then
    reaper.MB("File Exist","File Exist",0)
    goto start_dialog
 end   
 
 if retval and file ~= '' then
 
   if GetFileExtension(file) ~= ".mjb" then
      file = (file.. ".mjb")
   end  
   file_path = GetFilename(file)
 
   name_input = string.gsub(file_path, '[.]%w%w%w$','')
   --Msg("name_input ".. name_input)
   reaper.defer(Main)
 end
 
::finish::
--if not retval then 
  

 --file = (file.. ".mjb")
 
  
end
